#!/bin/bash
#
# build_pythia.sh

usage()
{
   echo "USAGE: `basename ${0}` <product_directory> <c7|gcc820|gcc930> <debug|prof> [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`
    
    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
    if [ -z ${SSIBUILDSHIMS_DIR} ]
    then
       echo "ERROR: failed to setup ssibuildshims ${ssibuildshims_version}"
       exit 1
    fi
}

set_qualifiers()
{
  qlist=(`echo ${basequal} | sed -e 's/:/ /g' | tr " " "\n"| sort |tr "\n" " "`)
  for qual in "${qlist[@]}"; do
    case ${qual} in
      gcc820|gcc930)
        cc=gcc
        fc=gfortran
        ;;
      c7)
        cc=clang
        fc=gfortran
        ;;
      *)
        ssi_die "Unrecognized qualifier ${qual}"
    esac
  done
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
basequal=${2}
extraqual=${3}
maketar=${4}

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=pythia
origpkgver=v6_4_28
pkgver=${origpkgver}q
pkgdotver=`echo ${origpkgver} | sed -e 's/_/./g' | sed -e 's/^v//'`
ssibuildshims_version=v2_00_00

get_this_dir

get_ssibuildshims

source define_basics --

if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
   exit 0
fi

echo "building ${package} for ${OS}-${plat}-${qualdir} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

set_qualifiers

if [ "${extraqual}" = "opt" ]
then
    extra_command="-DCMAKE_BUILD_TYPE=Release"
elif [ "${extraqual}" = "debug" ]
then
    extra_command="-DCMAKE_BUILD_TYPE=Debug"
elif [ "${extraqual}" = "prof" ]
then
    extra_command="-DCMAKE_BUILD_TYPE=RelWithDebInfo"
else
   echo "ERROR: please specify debug, opt, or prof"
   usage
   exit 1
fi

mkdir -p ${blddir}
if [ ! -d ${blddir} ]
then
   echo "ERROR: failed to create ${blddir}"
   exit 1
fi

# Remake this directory to avoid patch conflicts.
rm -rf ${sourcedir}/pythia
mkdir -p ${sourcedir}/pythia
if [ ! -d ${sourcedir}/pythia ]
then
   echo "ERROR: failed to create ${sourcedir}/pythia"
   exit 1
fi

# Fake ups declare.
fakedb=${product_dir}/${package}/${pkgver}/fakedb
${SSIBUILDSHIMS_DIR}/bin/fake_declare_product \
  ${product_dir} ${package} ${pkgver} ${fullqual} || \
  ssi_die "ERROR: fake declare failed"

for pdir in doc example tpythia6_build; do
  if [[ -d "${tardir}/${pdir}" ]] && [[ ! -d ${sourcedir}/${pdir} ]]; then
    echo "installing ${sourcedir}/${pdir}"
    mv ${tardir}/${pdir} ${sourcedir}/ || ssi_die "cannot move ${tardir}/${pdir}"
  fi
done
if [[ -f "${patchdir}/CMakeLists.txt" ]]; then
  mv ${patchdir}/CMakeLists.txt ${sourcedir}/ || ssi_die "cannot move ${patchdir}/CMakeLists.txt"
fi

cd ${sourcedir}/pythia || ssi_die "cannot cd to ${sourcedir}/pythia"
tar xf ${tardir}/pythia-${pkgdotver}.tgz || ssi_die "failed to unwind ${tardir}/pythia-${pkgdotver}.tgz"
patch -b -p0 < ${patchdir}/pythia6.patch || ssi_die "application of patch failed."


setup ${package} ${pkgver} -q ${fullqual} -z ${fakedb}:${product_dir}:${PRODUCTS} || ssi_die "ERROR: fake setup failed."

# We need cmake
source ${SSIBUILDSHIMS_DIR}/bin/setup_cmake || ssi_die "ERROR: failed to setup camke"

ncore=`${SSIBUILDSHIMS_DIR}/bin/ncores`

set -x

cd ${blddir}
CC=${cc} FC=${fc} \
 cmake -DCMAKE_INSTALL_PREFIX=${pkgdir} ${extra_command} ${sourcedir} || \
 ssi_die "ERROR: CMake failed."
make -j ${ncore} || ssi_die "ERROR: make failed."
ctest -j ${ncore} || ssi_die "ERROR: make test failed."
make install || ssi_die "ERROR: make install failed."

set +x

if [ ! -d ${pkgdir}/lib ]
then
   echo "ERROR: failed to create ${pkgdir}/lib"
   echo "       Something is very wrong"
   exit 1
fi

# real ups declare
${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
setup ${package} ${pkgver} -q ${fullqual} -z ${product_dir}:${PRODUCTS}
echo "pythia is installed at ${PYTHIA_FQ_DIR}"

# check ID and paths on OSX
if [ ${OS1} = "Darwin" ];then
  echo 
  echo "fix library rpaths"
  echo
  cd ${PYTHIA_FQ_DIR}/lib
  echo "before fixing"
  otool -L libPythia6.dylib
  fix_macos_lib_symbols libPythia6.dylib
  cd ${thisdir}
fi

# this must be last
if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
fi

exit 0
